"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitHub = void 0;
const Logger_1 = require("../../Logger");
const Parser_1 = require("../../Parser");
const filter_util_1 = require("../utils/filter.util");
const message_util_1 = require("../utils/message.util");
const CommitStatus_1 = require("./CommitStatus");
class GitHub {
    constructor(prService) {
        this.prService = prService;
        this.toCreateReviewComment = (commitSha, comment) => __awaiter(this, void 0, void 0, function* () {
            const { text, file, line } = comment;
            try {
                yield this.prService.createReviewComment(commitSha, text, file, line);
                Logger_1.Log.debug('GitHub create review success', { text, file, line });
                return comment;
            }
            catch (e) {
                // todo: this is workaround; handle comment on restrict zone in github
                const { name, status } = e !== null && e !== void 0 ? e : {};
                Logger_1.Log.warn('GitHub create review failed', {
                    comment,
                    error: { name, status },
                });
                return null;
            }
        });
    }
    report(logs) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.removeExistingComments();
                const commitId = yield this.prService.getLatestCommitSha();
                const touchedDiffs = yield this.prService.diff();
                const invalidLogs = logs.filter((l) => !l.valid);
                const touchedFileLog = logs
                    .filter(filter_util_1.onlyIn(touchedDiffs))
                    .filter(filter_util_1.onlySeverity(Parser_1.LogSeverity.error, Parser_1.LogSeverity.warning));
                Logger_1.Log.debug(`Commit SHA ${commitId}`);
                Logger_1.Log.debug('Touched diff', touchedDiffs);
                const comments = this.groupComments(touchedFileLog);
                const reviewResults = yield Promise.all(comments.map((comment) => this.toCreateReviewComment(commitId, comment)));
                const reviewedComments = reviewResults.filter((log) => !!log);
                Logger_1.Log.info(`Create ${reviewedComments.length} review comments. (with ${comments.length - reviewedComments.length} failed)`);
                const nOfErrors = reviewedComments.reduce((sum, comment) => sum + comment.errors, 0);
                const nOfWarnings = reviewedComments.reduce((sum, comment) => sum + comment.warnings, 0);
                if (nOfWarnings + nOfErrors > 0 || invalidLogs.length > 0) {
                    const overview = message_util_1.MessageUtil.generateOverviewMessage(nOfErrors, nOfWarnings);
                    const otherIssues = GitHub.createOtherIssue(invalidLogs);
                    yield this.prService.createComment(overview + (otherIssues ? `\n${otherIssues}` : ''));
                    Logger_1.Log.info('Create summary comment completed');
                }
                const commitStatus = nOfErrors ? CommitStatus_1.CommitStatus.failure : CommitStatus_1.CommitStatus.success;
                const description = message_util_1.MessageUtil.generateCommitDescription(nOfErrors);
                yield this.prService.createCommitStatus(commitId, commitStatus, description);
                Logger_1.Log.info('Report commit status completed');
            }
            catch (err) {
                Logger_1.Log.error('GitHub report failed', err);
                throw err;
            }
        });
    }
    static createOtherIssue(logs) {
        if (logs.length === 0)
            return null;
        const issuesTableContent = logs.map((l) => `| ${l.source} | ${l.msg} |`).join('\n');
        // Blank line required after </summary> to let markdown after it display correctly
        return `<details>
<summary><span>By the way, there are other issues those might not related to your code</span></summary>

| source | message |
|-|-|
${issuesTableContent}
</details>`;
    }
    removeExistingComments() {
        return __awaiter(this, void 0, void 0, function* () {
            const [userId, comments, reviews] = yield Promise.all([
                this.prService.getCurrentUserId(),
                this.prService.listAllComments(),
                this.prService.listAllReviewComments(),
            ]);
            Logger_1.Log.debug('Get existing CodeCoach comments completed');
            const deleteComments = comments
                .filter((comment) => { var _a; return ((_a = comment.user) === null || _a === void 0 ? void 0 : _a.id) === userId; })
                .map((comment) => this.prService.deleteComment(comment.id));
            const deleteReviews = reviews
                .filter((review) => { var _a; return ((_a = review.user) === null || _a === void 0 ? void 0 : _a.id) === userId; })
                .map((review) => this.prService.deleteReviewComment(review.id));
            yield Promise.all([...deleteComments, ...deleteReviews]);
            Logger_1.Log.debug('Delete CodeCoach comments completed');
        });
    }
    groupComments(logs) {
        const commentMap = logs.reduce((map, log) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j;
            const { source: file, line, severity, msg } = log;
            const text = message_util_1.MessageUtil.createMessageWithEmoji(msg, severity);
            if (!line)
                return map;
            const currentWarnings = (_c = (_b = (_a = map === null || map === void 0 ? void 0 : map[file]) === null || _a === void 0 ? void 0 : _a[line]) === null || _b === void 0 ? void 0 : _b.warnings) !== null && _c !== void 0 ? _c : 0;
            const currentErrors = (_f = (_e = (_d = map === null || map === void 0 ? void 0 : map[file]) === null || _d === void 0 ? void 0 : _d[line]) === null || _e === void 0 ? void 0 : _e.errors) !== null && _f !== void 0 ? _f : 0;
            const currentText = (_j = (_h = (_g = map === null || map === void 0 ? void 0 : map[file]) === null || _g === void 0 ? void 0 : _g[line]) === null || _h === void 0 ? void 0 : _h.text) !== null && _j !== void 0 ? _j : '';
            const nextObject = {
                text: `${currentText}\n${text}`,
                errors: currentErrors + (severity === Parser_1.LogSeverity.error ? 1 : 0),
                warnings: currentWarnings + (severity === Parser_1.LogSeverity.warning ? 1 : 0),
                file,
                line,
            };
            const fileCommentUpdater = { [line]: nextObject };
            const updatedFileComment = Object.assign({}, map === null || map === void 0 ? void 0 : map[file], fileCommentUpdater);
            const mapUpdater = { [file]: updatedFileComment };
            return Object.assign({}, map, mapUpdater);
        }, {});
        return Object.values(commentMap).flatMap((file) => Object.values(file));
    }
}
exports.GitHub = GitHub;
